#include "Header.hh"
#include <iostream>
#include <sstream>
#include <string>
#include <vector>

using std::string;
namespace Header
{

  SurveySchema::SurveySchema(SURVEY surveyType)
  {
    if(surveyType == S70){
      // This initialises the SurveyHeader arrays for the S70 setup.
      int s70Start[numKeywords] = {0,8,12,16,20,
				   28,36,44,60,
				   72,74,90,106,114,
				   602,602,122,130,138,
				   202,204,220,228,
				   252,276,284,292,602,300,
				   304,320,332,348,364,372,380,388,
				   396,602,428,492,500,508,
				   516,598,520,524,536,540,
				   556,560,568,576,578,584,586,
				   588,602,602,602,602};
      for(int i=0;i<numKeywords;i++) this->startPosition[i] = s70Start[i];
      
      int s70Length[numKeywords] = {8,4,4,4,8,
				    8,8,16,12,
				    2,16,16,8,8,
				    0,0,8,8,64,
				    2,8,4,12,
				    12,4,8,8,0,4,
				    16,12,16,16,8,8,8,8,
				    8,0,8,8,8,8,
				    4,4,4,12,4,16,
				    2,8,4,2,6,2,2,
				    10,0,0,0,38};
      for(int i=0;i<numKeywords;i++) this->fieldLength[i] = s70Length[i];

      int s70Dim[numKeywords] = {1,1,1,1,1,
				 1,1,1,1,
				 1,1,1,1,1,
				 1,1,1,1,1,
				 1,2,2,2,
				 2,2,1,1,1,1,
				 1,1,1,1,1,1,1,1,
				 4,1,8,1,1,1,
				 1,1,1,1,1,1,
				 2,1,2,1,1,1,1,
				 1,1,1,1,1};
      for(int i=0;i<numKeywords;i++) this->fieldDimension[i] = s70Dim[i];
    }
    else if(surveyType == PM || surveyType == PH){
      // This initialises the SurveyHeader arrays for the PM setup.
      int pmStart[numKeywords] = {0,6,10,16,20,
				  28,36,44,60,
				  72,74,90,106,114,
				  122,130,618,618,138,
				  202,204,220,228,
				  252,276,284,292,300,302,
				  304,320,332,348,364,372,380,388,
				  618,396,412,492,500,508,
				  618,516,520,524,618,540,
				  556,560,568,576,578,584,586,
				  588,598,606,614,618};
      for(int i=0;i<numKeywords;i++) this->startPosition[i] = pmStart[i];
      
      int pmLength[numKeywords] = {6,4,6,4,8,
				   8,8,16,12,
				   2,16,16,8,8,
				   8,8,0,0,64,
				   2,8,4,12,
				   12,4,8,8,2,1,
				   16,12,16,16,8,8,8,8,
				   0,4,4,8,8,8,
				   0,4,4,16,0,16,
				   2,8,4,2,6,2,2,
				   10,8,8,4,22};
      for(int i=0;i<numKeywords;i++) this->fieldLength[i] = pmLength[i];

      int pmDim[numKeywords] = {1,1,1,1,1,
				1,1,1,1,
				1,1,1,1,1,
				1,1,1,1,1,
				1,2,2,2,
				2,2,1,1,1,2,
				1,1,1,1,1,1,1,1,
				1,4,20,1,1,1,
				1,1,1,1,1,1,
				2,1,2,1,1,1,1,
				1,1,1,1,1};
      for(int i=0;i<numKeywords;i++) this->fieldDimension[i] = pmDim[i];
    }
//     else throw xBadSurvey(surveyType);
  }
  //--------------------------------------------------------------------------

  template <class T> 
  T SurveySchema::readParameter(string headerString, int field,
				int elementNumber)
  {
    int dim = this->fieldDimension[field];
    int length = this->fieldLength[field];
    int pos = this->startPosition[field];
    std::stringstream ss;
    std::vector<T> output(dim);
    for(int i=0;i<dim;i++){
      ss.str( headerString.substr(pos+elementNumber*length, length) );
      ss >> output[i];    
    }
    return output[elementNumber];
  }
  template <> string 
  SurveySchema::readParameter<string>(string headerString, 
				      int field, int elementNumber)
  {
    int dim = this->fieldDimension[field];
    int length = this->fieldLength[field];
    int pos = this->startPosition[field];
    return headerString.substr(pos+elementNumber*length, length);
  }
  template int SurveySchema::readParameter<int>(string headerString, 
						int field,
						int elementNumber);
  template float SurveySchema::readParameter<float>(string headerString, 
						    int field,
						    int elementNumber);
  template double SurveySchema::readParameter<double>(string headerString, 
						      int field,
						      int elementNumber);
  template string SurveySchema::readParameter<string>(string headerString, 
						    int field,
						    int elementNumber);
  //--------------------------------------------------------------------------



}
