#include <iostream>
#include <sstream>
#include <fitsio.h>
#include "../Utils.hh"
#include <vector>
#include <string>

int main()
{

//   std::vector<bool> bin(8,false);
//   bin[1] = bin[2] = bin[7] = true;
//   for(int i=0;i<8;i++) std::cout << bin[i];
//   std::cout << " --> " << binaryToString(bin) << std::endl;

  float version;
  fits_get_version(&version);
  std::cerr << "Version no." << version << std::endl;

  fitsfile *fptr;
  int status = 0;
  char *template_file = (char *)getPSRFITSdefn().c_str();
  std::cerr << "opening with template...";
  //  fits_create_template(&fptr, "!test.fits", getPSRFITSdefn().c_str(),&status);
  //  fits_create_template(&fptr, "!test.fits", "simpletemplate.fits",&status);
//   fits_create_template(&fptr,"!test.fits","combinedtemplate.fits",&status);
  fits_create_template(&fptr,"!test.fits","psrheaderSimple.txt",&status);
  if(status) fits_report_error(stderr, status);
  std::cerr << "done.\n";

  std::cerr << "moving to subint hdu...";
  status = 0;
  fits_movnam_hdu(fptr, ANY_HDU, "subint", 0, &status);
  if(status) fits_report_error(stderr, status);
  std::cerr << "done.\n";

  int nch=16;
  int nbin=8;
  int npol=1;

  status=0;
  std::cerr << "updating keywords...";
  fits_update_key(fptr, TINT, "NCH_FILE", &nch, NULL, &status);
  if(status) fits_report_error(stderr, status);
  int nch_strt=0;
  status=0;
  fits_update_key(fptr, TINT, "NCH_STRT", &nch_strt, NULL, &status);
  if(status) fits_report_error(stderr, status);
  std::cerr << "done.\n";
  
  std::cerr << "fixing format...";
  std::stringstream ss;
  int colnum;
  status=0;
  fits_get_colnum(fptr, CASEINSEN, "DATA", &colnum, &status);
  if(status) fits_report_error(stderr, status);
  ss << "TFORM" << colnum;
  char *tform = (char*)ss.str().c_str();
  char cval[5];
  strcpy(cval,"9X");
//   strcpy(cval,"2B");
  status=0;
  std::cerr << " changing " << tform << " to \"" << cval <<"\"...";
  fits_update_key(fptr, TSTRING, tform, cval, NULL, &status);
  if(status) fits_report_error(stderr, status);
//   status=0;
//   fits_compress_heap(fptr,&status);
//   if(status) fits_report_error(stderr, status);
  std::cerr << "done.\n";

  std::cerr << "moving to primary hdu...";
  status = 0;
  fits_movabs_hdu(fptr, 1, NULL, &status);    
  if(status) fits_report_error(stderr, status);
  std::cerr << "done.\n";
  std::cerr << "moving to subint hdu...";
  status = 0;
  fits_movnam_hdu(fptr, ANY_HDU, "subint", 0, &status);
  if(status) fits_report_error(stderr, status);
  std::cerr << "done.\n";

  long nrows;
  std::cerr << "getting number of rows...";
  status=0;
  fits_get_num_rows(fptr, &nrows, &status);
  if(status) fits_report_error(stderr, status);
  std::cerr << "num rows=" << nrows << std::endl;
  status=0;

  // Add info in the TSUBINT column in the current row
//   std::cerr << "setting tsubint column...";
//   status=0;
//   double tsubint = 100.;
//   fits_get_colnum(fptr, CASEINSEN, "TSUBINT", &colnum, &status);
//   if(status) fits_report_error(stderr, status);  
//   status=0;
//   fits_write_col(fptr, TDOUBLE, colnum, nrows+1, 1, 1, &tsubint, &status);
//   if(status) fits_report_error(stderr, status);
//   std::cerr << "done.\n";
  
  std::cerr << "adding data array...";
  std::vector<bool> bin(nch*nbin*npol,false);
  for(int i=0;i<nbin;i++) 
    bin[i*nch+2] = bin[i*nch+5] = bin[i*nch+9] = true;
  for(int i=0;i<bin.size();i++) std::cerr << bin[i];
  std::cerr << std::endl;
  std::string binstring = binaryToString(bin);
  unsigned char *data = new unsigned char[nch*npol*nbin/8];
//   std::cerr << "data=\"";
  for(int i=0;i<binstring.size();i++){
    data[i] = binstring[i];
//     std::cerr << data[i];
  }
//   std::cerr << "\"\n";
//   unsigned char *data = new unsigned char[nch*npol*nbin];
// //   char *data = new char[nch*npol*nbin];
//   for(int i=0;i<bin.size();i++){
//     if(bin[i]) data[i] = 1;
//     else data[1]=0;
//   }
  unsigned int *dataInt = binaryToInt(bin);
  for(int i=0;i<nch*nbin*npol/8;i++) std::cerr<<dataInt[i]<< " ";
  std::cerr<<"\n";

  status=0;
  fits_get_colnum(fptr, CASEINSEN, "DATA", &colnum, &status);
  if(status) fits_report_error(stderr, status);
  /* */
  
  std::cerr << "fixing size of vector...";
  status=0;
//   fits_modify_vector_len (fptr, colnum, nch*npol*nbin/8, &status);
  fits_modify_vector_len (fptr, colnum, nch*npol*nbin, &status);
  if(status) fits_report_error(stderr, status);
  std::cerr << "done.\n";
  std::cerr << "fixing dimensions...";
  long naxis=3,naxes[3];
//   naxes[0] = nbin; naxes[1] = nch/8; naxes[2] = npol;
  naxes[0] = nch; naxes[1] = npol; naxes[2] = nbin; 
  /* */
  status=0;
  fits_write_tdim(fptr, colnum, naxis, naxes, &status);
  if(status) fits_report_error(stderr, status);
  std::cerr << "done.\n";

  std::cerr << "writing data...";
  status=0;
//   fits_write_col(fptr, TBIT, colnum, nrows+1, 1, nch*npol*nbin, &data, &status);
  //fits_write_col_bit(fptr, colnum, nrows+1, 1, nch*npol*nbin, data, &status);
//   fits_write_col(fptr, TBYTE, colnum, nrows+1, 1, nch*npol*nbin/8, &dataInt, &status);
//   fits_write_col(fptr, TBYTE, colnum, nrows+1, 1, nch*npol*nbin, &data, &status);
  fits_write_col_byt(fptr, colnum, nrows+1, 1, nch*npol*nbin/8, data, &status);
  if(status) fits_report_error(stderr, status);
  std::cerr << "done.\n";
  /* */
  std::cerr << "closing...";
  status = 0;
  fits_close_file(fptr, &status);
  if (status){
    fits_report_error(stderr, status);
  }
  std::cerr << "done.\n";

}
