#include <iostream>
#include <sstream>
#include <fstream>
#include <iomanip>
#include <vector>
#include <string>
using namespace std;

enum FIELDS 
{PROGRAM=0, VERSION, TAPENR, FILE_CNTR, BLK_CNTR, DATE, MJD, UT_STRT, LST_STRT, COORD_TYPE,
 RA_STRT, DEC_STRT, GL_STRT, GB_STRT, FEED_ANGLE, OBS_TIME, COMMENT, NFILTER, CHANBW, NCHAN, FREQ_CHAN1,
 SAMP_INT, SAMP_GRP, GRP_BLK, BLK_SEC, FDCNTRL, DATA, UT, LST, RA, DEC, GL, GB, TEL_ZEN,
 TEL_AZ, ATTEN, TPOWER, NBLK_READ, SCAN_RATEL, SCAN_RATEB, NBEAM, IBEAM, PNAME, OBS_FILEN, NBITS, DDM, NDDCH, MOVE,
 PNTERR, TREE, NSYS, TELID, PANGLE, SMP_BSW, CAL_CYCLE, RESERVE};

const string fieldName[RESERVE+1] = 
{"PROGRAM", "VERSION", "TAPENR", "FILE_CNTR", "BLK_CNTR", "DATE", "MJD", "UT_STRT", "LST_STRT", "COORD_TYPE",
" RA_STRT", "DEC_STRT", "GL_STRT", "GB_STRT", "FEED_ANGLE", "OBS_TIME", "COMMENT", "NFILTER", "CHANBW", "NCHAN", "FREQ_CHAN1",
" SAMP_INT", "SAMP_GRP", "GRP_BLK", "BLK_SEC", "FDCNTRL", "DATA", "UT", "LST", "RA", "DEC", "GL", "GB", "TEL_ZEN",
" TEL_AZ", "ATTEN", "TPOWER", "NBLK_READ", "SCAN_RATEL", "SCAN_RATEB", "NBEAM", "IBEAM", "PNAME", "OBJS_FILEN", "NBITS", "DDM", "NDDCH", "MOVE",
" PNTERR", "TREE", "NSYS", "TELID", "PANGLE", "SMP_BSW", "CAL_CYCLE", "RESERVE"};

const int fieldDim[RESERVE+1]={1,1,1,1,1,1,1,1,1,1,
			 1,1,1,1,1,1,1,1,2,2,2,
			 2,2,1,1,1,2,1,1,1,1,1,1,1,
			 1,4,20,1,1,1,1,1,1,1,2,1,2,1,
			 1,1,1,1,1,1,1,1};

const int fieldSize[RESERVE+1] = {6,4,6,4,8,8,8,16,12,2,
			    16,16,8,8,8,8,64,2,8,4,12,
			    12,4,8,8,2,1,16,12,16,16,8,8,8,
			    8,4,4,8,8,8,4,4,16,16,2,8,4,2,
			    6,2,2,10,8,8,4,22};

int totalFieldSize[RESERVE+1];
int fieldStartLoc[RESERVE+1];

inline string getSub(string hdr, int field)
{return hdr.substr(fieldStartLoc[field],totalFieldSize[field]);};
			 

int main(int argc, char * argv[])
{
  /**
   *  An attempt to read the binary data from a PTTAPE file
   *  and write it out in some sensible format...
   */

  ifstream fin;
  string filename;
  if(argc==1){ 
    filename = "../data/pttape_test";
  }
  else if(argc==2){
    filename = argv[1];
  }
  else{
    cerr << "Either \"pm.x\" or \"pm.x filename\"...\n";
    return(1);
  }

  const int HDR_LENGTH=640;     // length of the header in bytes
  const int DAT_LENGTH=48*1024; // length of the data field in bytes

  for(int i=0;i<=RESERVE;i++) totalFieldSize[i] = fieldDim[i] * fieldSize[i];
  fieldStartLoc[0] = 0;
  for(int i=1;i<=RESERVE;i++) fieldStartLoc[i] = fieldStartLoc[i-1] + totalFieldSize[i-1];

  fin.open(filename.c_str(),ios::binary);
  //   // define and read in the header
  char *hdr = new char[HDR_LENGTH];
  fin.read( hdr, HDR_LENGTH*sizeof(char) );
  string hdrstring = hdr;

  std::cerr << "Header field:\n" << hdr << std::endl;
  fin.close();

  std::cout << "Header parsing:\n";
  for(int i=0; i<=RESERVE; i++){
    std::cout << setw(10) << fieldName[i] << ":\t";
    //    std::cout << fieldStartLoc[i] << std::endl;
    std::cout << "[" << setw(3) << fieldStartLoc[i] << "]:\t";
//     std::cout << hdrstring.substr(fieldStartLoc[i],totalFieldSize[i]) << std::endl;
    if(getSub(hdrstring,i)[0]!=0) std::cout << getSub(hdrstring,i) << std::endl;
  }


  fin.open(filename.c_str(),ios::binary);
  // now try and read in the data
  //  char *data = new char[DAT_LENGTH]; // define data as bool type, which is 
  vector<char> data;
  char newone;
//   while(!fin.eof()) {
//   for(int i=0;i<DAT_LENGTH;i++){
//     fin.read( &newone, sizeof(char) );
//     data.push_back(newone);
//   }

//   std::cerr << "Data field -- size = " << data.size() << ":\n";
//   for(int i=0;i<data.size();i++) 
// //     std::cerr << atoi(&data[i]) << "|" << int(data[i]) <<" ";
//     if(atoi(&data[i])!=0) 
//       std::cerr << atoi(&data[i]) << "|" << int(data[i]) <<"|" << data[i] << " ";
//   std::cerr << std::endl;
  
  string fullstring, oldtime = "bob",time;
  vector<int> changes;
  vector<float> seconds;
  for(int loop=0;loop<500;loop++){
    fin.read( hdr, HDR_LENGTH*sizeof(char) );
    fullstring = hdr;

    float blk_sec = atof(getSub(fullstring,BLK_SEC).c_str());
    int grp_blk = atoi( getSub(fullstring,GRP_BLK).c_str() );
    stringstream ss;
    ss.str(getSub(fullstring,NCHAN));
    int nch[2];
    ss >> nch[0] >> nch[1];
    time = getSub(fullstring,UT).c_str();
    if(oldtime != time){
      std::cerr << setw(3) << loop+1 << ":"
		<< setw(totalFieldSize[BLK_CNTR]) <<getSub(fullstring,BLK_CNTR)
		<< setw(totalFieldSize[UT]) << time
		<< setw(totalFieldSize[RA_STRT]) << getSub(fullstring,RA_STRT)
		<< setw(totalFieldSize[DEC_STRT]) << getSub(fullstring,DEC_STRT)
		<< setw(totalFieldSize[RA]) << getSub(fullstring,RA)
		<< setw(totalFieldSize[DEC]) << getSub(fullstring,DEC)
		<< setw(totalFieldSize[MOVE]) << getSub(fullstring,MOVE)
		<< setw(totalFieldSize[IBEAM]) << getSub(fullstring,IBEAM)
		<< " " << blk_sec / float(grp_blk) 
		<< " " << 48.*1024.*8. / (nch[0] + nch[1])
		<< " " << 48.*1024.*8. / (nch[0] + nch[1]) * blk_sec / float(grp_blk) 
		<< " " << setw(totalFieldSize[MOVE]) << getSub(fullstring,MOVE)
		<< std::endl;
      changes.push_back(loop+1);
      seconds.push_back(atof( fullstring.substr(311,8).c_str() ));
    }
      oldtime = time;
    for(int i=0;i<DAT_LENGTH;i++){
      fin.read( &newone, sizeof(char) );
      data.push_back(newone);
    }
  }

  cerr << endl << endl;

  for(int i=0;i<changes.size();i++) cerr << setw(3) << changes[i] << " ";
  cerr << endl;
  for(int i=1;i<changes.size();i++) cerr << setw(3) << changes[i] - changes[i-1]<< " ";
  cerr << endl;
  for(int i=1;i<seconds.size();i++) 
    cerr << setw(7) << fmodf(seconds[i] - seconds[i-1], 60.)<< " ";
  cerr << endl;

}

