#include <iostream>
#include <iomanip>
#include <fstream>
#include <sstream>
#include <string>
#include "../Block.hh"
#include "../Header.hh"
#include "../Utils.hh"

using namespace Header;
using namespace Block;
using std::string;


int main()
{
  string fieldnames[numKeywords] = 
    {"CODE", "VERSION", "TAPENR", "SCAN_CNTR", "BLK_CNTR", 
     "DATE", "MJD", "UT_STRT", "LST_STRT", 
     "COORD_TYPE", "RA_STRT", "DEC_STRT", "GL_STRT", "GB_STRT", 
     "FEED_ANGLE", "OBS_TIME", "SCAN_RATE", "SCAN_LEN", "COMMENT", 
     "NFILTER", "CHANBW", "NCHAN", "FREQ_CHAN1", 
     "SAMP_INT", "SAMP_GRP", "GRP_BLK", "BLK_SEC", "FDCNTRL", "DATA_TYPE", 
     "UT", "LST", "RA", "DEC", "GL", "GB", "TEL_ZEN", "TEL_AZ", 
     "STATUS", "ATTEN", "TPOWER", "NBLK_READ", "SCAN_RATEL", "SCAN_RATEB", 
     "BYTE_GRP", "NBEAM", "IBEAM", "PSRNAME", "DATA_TWO", "OBS_FILEN", 
     "NBITS", "DDM", "NDDCH", "MOVE", "PNTERR", "TREE", "NSYS", 
     "TELID", "PANGLE", "SMP_BSW", "CAL_CYCLE", "RESERVE"};

  string surveylist[3]={"S70","PM","bob"};
  string filelist[3]={"data/s70_tape_test_S181_S18101.dat",
		      "data/pttape_test","data/pttape_test"};

  try{
    
    for(int s=0;s<3;s++){
      
      std::cout <<surveylist[s]<<":\n";
      
      DataBlock block(surveylist[s]);
      std::ifstream fin;
      fin.open(filelist[s].c_str(),std::ios::binary);
      block.readHeader(fin);

      for(int i=0;i<numKeywords;i++){
	std::cout << std::setw(15) << fieldnames[i]+":";
	std::cout << "[" << std::setw(3) << block.Survey().getStartPosition(i)
		  << "," << std::setw(2) << block.Survey().getFieldLength(i)
		  << "," << std::setw(2) << block.Survey().getFieldDimension(i)
		  << "]\t";
	for(int d=0;d<block.Survey().getFieldDimension(i);d++){
	  if(d>0) std::cout << ",";
	  std::cout << "\""<<removeLeadingBlanks(block.getKeyword<string>(i,d))<<"\"";
	}
	std::cout <<"\n";
      }

      block.readData(fin);
      std::cout <<"\n";
      block.readHeader(fin);

      for(int i=0;i<numKeywords;i++){
	std::cout << std::setw(15) << fieldnames[i]+":";
	std::cout << "[" << std::setw(3) << block.Survey().getStartPosition(i)
		  << "," << std::setw(2) << block.Survey().getFieldLength(i)
		  << "," << std::setw(2) << block.Survey().getFieldDimension(i)
		  << "]\t";
	for(int d=0;d<block.Survey().getFieldDimension(i);d++){
	  if(d>0) std::cout << ",";
	  std::cout << "\""<<removeLeadingBlanks(block.getKeyword<string>(i,d))<<"\"";
	}
	std::cout <<"\n";
      }
      std::cout <<"\n\n\n";
      fin.close();
    }
  }
  catch(SurveySchema::xBadSurvey bad){
    std::cerr << "Survey \"" << bad.getSurvey() << "\" does not exist!\n";
  }

  char test = 'K'; // =75;
  string testBinary = charToBinary(test);
  std::cout << test << " = " << int(test) << " = " << testBinary <<"\n";

}
